package plugins.SamplePlugin;

import info.clearthought.layout.TableLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URLDecoder;
import java.util.Locale;
import java.util.TreeMap;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import allrounder.DbHandler;
import allrounder.DirHandler;
import allrounder.StringLib;
import allrounder.plugin.YiffIndexPlugin;

// All plugins have to implement the YiffIndexPlugin interface. You can do everything else pretty much as you wish
// Note that you can access all of YiffIndex' functions and classes from within your plugin
public class SamplePlugin implements YiffIndexPlugin {
	TreeMap<String, String> mLanguageMap = null;

	public SamplePlugin() {
		// only do stuff here that you absolutely must have for the interface functions getName() and getDescription()
		// if you only have one supported language for your plugin, this should probably be empty
		// keep this as small as possible - it's called every time the pluginbrowser gets opened
		
		// To load the plugin language file(s) (in plugins/SamplePlugin). Note that the default language
		// (in this case English) has to exist no matter what because it's used when no language file
		// in the system language is present. If you support only one language anyway, you can use it
		// directly in the program without bothering to use StringLib
		// PS: Your language files have to be formatted the same way as the YiffIndex ones:
		// descriptor=text in the actual language
		// and have to use the short language name + .lng (for example, en.lng) as the file name.
		// For a list of all short language names, see the file "list of language codes.txt" in YiffIndex's language directory
		// PPS: DirHandler.getDirOfClass(getClass()) gets the directory of our class file, which is the same as the one
		// for the language files
		mLanguageMap = StringLib.getLanguageMapBySystemLocale(DirHandler.getDirOfClass(getClass()), Locale.ENGLISH);
	}

	public String getName() {
		// return the name of your plugin (no linebreaks). Keep this as short as possible
		return mLanguageMap.get("title");
	}

	public String getDescription() {
		// return the description of your plugin. Linebreaks possible using \n (note: if you use
		// StringLib, all your \n in your language file were read as text, e.g. \\n - you have to revert that first
		
		// recreating \n from languagefile
		String s = mLanguageMap.get("description");
		while (s.contains("\\n")) s = s.substring(0, s.indexOf("\\n")) + "\n" + s.substring(s.indexOf("\\n") + 2);
		return s;
	}

	// this function is called every time the Plugin is executed. You could put all code of the
	// plugin here, but I would just make a new class for all of it and just call
	// the constructor of it here, like it is done in this example
	// Just treat this method the way you would treat the main method in other apps.
	// the frame in the parameter is just the main frame of YiffIndex, intended to get the positioning
	// of your dialog(s) right. Just ignore it if you don't need it.
	public void pluginCalled(JFrame mainFrame) {
		new DisplayNumberOfIndexedImages(mLanguageMap, mainFrame);
	}
}

// It's probably the best way to do all the real stuff your plugin does in an extra class
// That way, you can rely on constructors and destructors
// This sample class is really simple - it just uses some of YiffIndex' classes to display the number
// of total indexed images
class DisplayNumberOfIndexedImages extends JDialog {
	TreeMap<String, String> mLanguageMap = null;
	
	JLabel lblNumOfImagesText = new JLabel();
	JLabel lblNumOfImagesNumber = new JLabel();
	JButton btnClose = new JButton();
	
	DisplayNumberOfIndexedImages(TreeMap<String, String> languageMap, JFrame mainFrame) {
		if ((mLanguageMap = languageMap) == null) return;

		lblNumOfImagesText.setText(mLanguageMap.get("numOfImages"));
		lblNumOfImagesNumber.setText(getNumberOfImagesAsString());
		btnClose.setText(mLanguageMap.get("close"));
		
		btnClose.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				setVisible(false);
				dispose();
			}
		});
		
		setTitle(mLanguageMap.get("Dialog-title"));
		((JPanel)this.getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		setSize(320, 100);
		setLocationRelativeTo(mainFrame);
		setResizable(true);
		
		// note that I used TableLayout here, The Best Layout Manager Ever[TM]. It's not a default java layout manager, but
		// because I used it in YiffIndex, I can use it here. If you want to use a normal LayoutManager, feel free to do so.
		// and yes, I know that TableLayout doesn't make too much sense for such a small dialog.
		//column / row
		double[][] layout = {{lblNumOfImagesText.getPreferredSize().width, 10, TableLayout.FILL},
							{TableLayout.FILL, 10, btnClose.getPreferredSize().height}};
		this.setLayout(new TableLayout(layout));

		this.add(lblNumOfImagesText, "0, 0");
		this.add(lblNumOfImagesNumber, "2, 0, c, c");
		this.add(btnClose, "0, 2");
		
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setVisible(true);
	}

	// does exactly what it sounds like
	private String getNumberOfImagesAsString() {
		// uses getAllImageIds() instead of getAllImages() because it's a bit faster
		// and uses less memory
		int iNumOfIndexedImages = DbHandler.getInstance().getAllImageIds().size();
		return Integer.toString(iNumOfIndexedImages);
	}
}
